@@ -3,9 +3,10 @@ author: Etienne Wallet
 
 Errors used in the MxOps package
 """
+
 from dataclasses import dataclass
 from pathlib import Path
-from typing import Any, List
+from typing import Any, Dict, List
 
 from multiversx_sdk_network_providers.transactions import TransactionOnNetwork
 
@@ -24,14 +25,25 @@ class ParsingError(Exception):
     To be raise when some data could not be parsed successfuly
     """
 
-    def __init__(self, raw_object: Any, parsing_target: str, ) -> None:
+    def __init__(
+        self,
+        raw_object: Any,
+        parsing_target: str,
+    ) -> None:
         message = f"Could not parse {raw_object} as {parsing_target}"
         super().__init__(message)
 
 
+class NewTokenIdentifierNotFound(Exception):
+    """
+    To be raised when the token identifier of newly issued token was not found in
+    the results of the transaction
+    """
+
+
 #############################################################
 #
-#                   Data Managment Errors
+#                   Data Management Errors
 #
 #############################################################
 
@@ -42,7 +54,7 @@ class UnknownScenario(Exception):
     """
 
     def __init__(self, scenario_name: str) -> None:
-        message = f'Scenario {scenario_name} is unkown'
+        message = f"Scenario {scenario_name} is unkown"
         super().__init__(message)
 
 
@@ -52,7 +64,7 @@ class UnloadedScenario(Exception):
     """
 
     def __init__(self) -> None:
-        message = 'Scenario data was not loaded'
+        message = "Scenario data was not loaded"
         super().__init__(message)
 
 
@@ -62,8 +74,7 @@ class UnknownContract(Exception):
     """
 
     def __init__(self, scenario_name: str, contract_id: str) -> None:
-        message = (f'Contract {contract_id} is unkown in '
-                   f'scenario {scenario_name}')
+        message = f"Contract {contract_id} is unkown in " f"scenario {scenario_name}"
         super().__init__(message)
 
 
@@ -73,7 +84,7 @@ class UnknownAccount(Exception):
     """
 
     def __init__(self, account_name: str) -> None:
-        message = f'Account {account_name} is unkown in the current scene'
+        message = f"Account {account_name} is unkown in the current scene"
         super().__init__(message)
 
 
@@ -83,7 +94,37 @@ class ContractIdAlreadyExists(Exception):
     """
 
     def __init__(self, contract_id: str) -> None:
-        message = f'Contract id {contract_id} already exists'
+        message = f"Contract id {contract_id} already exists"
+        super().__init__(message)
+
+
+class UnknownToken(Exception):
+    """
+    To be raised when a specified token is not found is a scenario
+    """
+
+    def __init__(self, scenario_name: str, token_name: str) -> None:
+        message = f"Token named {token_name} is unkown in " f"scenario {scenario_name}"
+        super().__init__(message)
+
+
+class TokenNameAlreadyExists(Exception):
+    """
+    To be raised when there is a conflict with token name
+    """
+
+    def __init__(self, token_name: str) -> None:
+        message = f"Token named {token_name} already exists"
+        super().__init__(message)
+
+
+class UnknownRootName(Exception):
+    """
+    To be raised when a specified root name is not found is a scenario
+    """
+
+    def __init__(self, scenario_name: str, root_name: str) -> None:
+        message = f"Root named {root_name} is unkown in " f"scenario {scenario_name}"
         super().__init__(message)
 
 
@@ -93,7 +134,7 @@ class ScenarioNameAlreadyExists(Exception):
     """
 
     def __init__(self, scenario_name: str) -> None:
-        message = f'Scenario name {scenario_name} already exists'
+        message = f"Scenario name {scenario_name} already exists"
         super().__init__(message)
 
 
@@ -104,8 +145,10 @@ class WrongScenarioDataReference(Exception):
     """
 
     def __init__(self) -> None:
-        message = ('Scenario data reference must have the format '
-                   r'"%contract_id%valuekey[:optional_format]"')
+        message = (
+            "Scenario data reference must have the format "
+            r'"%<value_key>[:optional_format]"'
+        )
         super().__init__(message)
 
 
@@ -115,11 +158,14 @@ class ForbiddenSceneNetwork(Exception):
     a network that the scene does not allow
     """
 
-    def __init__(self, scene_path: Path, network_name: str, allowed_networks: List[str]) -> None:
-        message = (f'Scene {scene_path} not allowed to be executed '
-                   f'in the network {network_name}.\n'
-                   f'Allowed networks: {allowed_networks}'
-                   )
+    def __init__(
+        self, scene_path: Path, network_name: str, allowed_networks: List[str]
+    ) -> None:
+        message = (
+            f"Scene {scene_path} not allowed to be executed "
+            f"in the network {network_name}.\n"
+            f"Allowed networks: {allowed_networks}"
+        )
         super().__init__(message)
 
 
@@ -129,13 +175,34 @@ class ForbiddenSceneScenario(Exception):
     a scenario that the scene does not allow
     """
 
-    def __init__(self, scene_path: Path, scenario_name: str, allowed_scenario: List[str]) -> None:
-        message = (f'Scene {scene_path} not allowed to be executed '
-                   f'in the scenario {scenario_name}.\n'
-                   f'Allowed scenario: {allowed_scenario}'
-                   )
+    def __init__(
+        self, scene_path: Path, scenario_name: str, allowed_scenario: List[str]
+    ) -> None:
+        message = (
+            f"Scene {scene_path} not allowed to be executed "
+            f"in the scenario {scenario_name}.\n"
+            f"Allowed scenario: {allowed_scenario}"
+        )
         super().__init__(message)
 
+
+class WrongDataKeyPath(Exception):
+    """
+    To be raised when a key path does not correspond to the saved
+    data
+    """
+
+
+class NoDataForContract(Exception):
+    """
+    To be raised when a specified contract has no data saved (analyze submodule)
+    """
+
+    def __init__(self, contract_bech32_address: str) -> None:
+        message = f"Contract {contract_bech32_address} has no saved data"
+        super().__init__(message)
+
+
 #############################################################
 #
 #                   Transactions Errors
@@ -154,7 +221,7 @@ class TransactionError(Exception):
         super().__init__()
 
     def __str__(self) -> str:
-        return f'Error on transaction {get_tx_link(self.tx.hash)}\n'
+        return f"Error on transaction {get_tx_link(self.tx.hash)}\n"
 
 
 class FailedTransactionError(TransactionError):
@@ -203,21 +270,84 @@ class EmptyQueryResults(Exception):
     """
 
 
+class QueryFailed(Exception):
+    """
+    To be raised when a query failed
+    """
+
+
 #############################################################
 #
 #                   Check Errors
 #
 #############################################################
 
+
 class CheckFailed(Exception):
     """
     To be raised when an on-chain transaction check fails
     """
 
-    def __init__(self, check: dataclass, tx: TransactionOnNetwork, *args: object) -> None:
+    def __init__(self, check: dataclass, tx: TransactionOnNetwork) -> None:
         self.check = check
         self.tx = tx
-        super().__init__(*args)()
+        super().__init__()
 
     def __str__(self) -> str:
-        return f'Check failed on transaction {get_tx_link(self.tx.hash)}\nCheck: {self.check}'
+        return (
+            f"Check failed on transaction {get_tx_link(self.tx.hash)}"
+            f"\nCheck: {self.check}"
+        )
+
+
+#############################################################
+#
+#                   User Errors
+#
+#############################################################
+
+
+class UnkownStep(Exception):
+    """
+    to be raised when the user provide a step name that is unkown
+    """
+
+    def __init__(self, step_name: str) -> None:
+        self.step_name = step_name
+        super().__init__()
+
+    def __str__(self) -> str:
+        return f"Unkown Step name: {self.step_name}"
+
+
+class UnkownVariable(Exception):
+    """
+    to be raised when the user provide a variable name that is unkown
+    """
+
+    def __init__(self, var_name: str) -> None:
+        self.step_name = var_name
+        super().__init__()
+
+    def __str__(self) -> str:
+        return f"Unkown variable: {self.step_name}"
+
+
+class InvalidStepDefinition(Exception):
+    """
+    to be raised when the arguments provided by the user for a Step are not valid
+    """
+
+    def __init__(self, step_name: str, parameters: Dict) -> None:
+        self.step_name = step_name
+        self.parameters = parameters
+        super().__init__()
+
+    def __str__(self) -> str:
+        return f"Step {self.step_name} received invalid parameters {self.parameters}"
+
+
+class InvalidQueryResultsDefinition(Exception):
+    """
+    to be raise when the results types of a query are not correctly defined
+    """
