@@ -7,19 +7,20 @@ from dataclasses import dataclass, field
 import os
 from pathlib import Path
 import re
-from typing import Dict, List
+from typing import Dict, List, Union
 
+from mxpyserializer.abi_serializer import AbiSerializer
 import yaml
 
 from mxops.config.config import Config
-from mxops.data.data import ExternalContractData, ScenarioData
-from mxops.execution.steps import Step, instanciate_steps
+from mxops.data.execution_data import _ScenarioData, ExternalContractData, ScenarioData
+from mxops.execution.steps import LoopStep, SceneStep, Step, instanciate_steps
 from mxops.execution.account import AccountsManager
 from mxops import errors
 from mxops.utils.logger import get_logger
 
 
-LOGGER = get_logger('scene')
+LOGGER = get_logger("scene")
 
 
 @dataclass
@@ -28,11 +29,14 @@ class Scene:
     Dataclass to represent a set of step to execute sequentially
     within a scenario.
     """
+
     allowed_networks: List[str]
     allowed_scenario: List[str]
     accounts: List[Dict] = field(default_factory=list)
     steps: List[Step] = field(default_factory=list)
-    external_contracts: Dict[str, str] = field(default_factory=dict)
+    external_contracts: Dict[str, Union[str, Dict[str, str]]] = field(
+        default_factory=dict
+    )
 
     def __post_init__(self):
         """
@@ -53,7 +57,7 @@ def load_scene(path: Path) -> Scene:
     :return: _description_
     :rtype: List[Step]
     """
-    with open(path.as_posix(), 'r', encoding='utf-8') as file:
+    with open(path.as_posix(), "r", encoding="utf-8") as file:
         raw_scene = yaml.safe_load(file)
 
     return Scene(**raw_scene)
@@ -66,7 +70,7 @@ def execute_scene(scene_path: Path):
     :param scene_path: path to the scene file
     :type scene_path: Path
     """
-    LOGGER.info(f'Executing scene {scene_path}')
+    LOGGER.info(f"Executing scene {scene_path}")
     scene = load_scene(scene_path)
     scenario_data = ScenarioData.get()
 
@@ -74,11 +78,12 @@ def execute_scene(scene_path: Path):
     network = config.get_network()
 
     # check network authorization
-    if network.name not in scene.allowed_networks:
+    if (
+        network.name not in scene.allowed_networks
+        and network.value not in scene.allowed_networks
+    ):
         raise errors.ForbiddenSceneNetwork(
-            scene_path,
-            network.name,
-            scene.allowed_networks
+            scene_path, network.value, scene.allowed_networks
         )
 
     # check scenario authorizations
@@ -89,27 +94,57 @@ def execute_scene(scene_path: Path):
             break
     if not match_found:
         raise errors.ForbiddenSceneScenario(
-            scene_path,
-            scenario_data.name,
-            scene.allowed_scenario
+            scene_path, scenario_data.name, scene.allowed_scenario
         )
 
     # load accounts
     for account in scene.accounts:
         AccountsManager.load_account(**account)
-        AccountsManager.sync_account(account['account_name'])
+        AccountsManager.sync_account(account["account_name"])
 
     # load external contracts addresses
-    for contract_id, address in scene.external_contracts.items():
+    for contract_id, contract_data in scene.external_contracts.items():
+        if isinstance(contract_data, str):
+            contract_data = {"address": contract_data}
+        address = contract_data["address"]
         try:
-            # try to update the contract address while keeping data intact
-            scenario_data.set_contract_value(contract_id, 'address', address)
+            serializer = AbiSerializer.from_abi(Path(contract_data["abi_path"]))
+        except KeyError:
+            serializer = None
+        try:
+            scenario_data.set_contract_value(contract_id, "address", address)
+            scenario_data.contracts_data[contract_id].serializer = serializer
         except errors.UnknownContract:
             # otherwise create the contract data
-            scenario_data.add_contract_data(ExternalContractData(contract_id, address, {}))
+            scenario_data.add_contract_data(
+                ExternalContractData(
+                    contract_id=contract_id,
+                    address=address,
+                    serializer=serializer,
+                    saved_values={},
+                )
+            )
 
     # execute steps
     for step in scene.steps:
+        execute_step(step, scenario_data)
+
+
+def execute_step(step: Step, scenario_data: _ScenarioData):
+    """
+    Execute a step
+
+    :param step: step to execute
+    :type step: Step
+    :param scenario_data: data of the current Scenario
+    :type scenario_data: _ScenarioData
+    """
+    if isinstance(step, SceneStep):
+        execute_scene(Path(step.scene_path))
+    elif isinstance(step, LoopStep):
+        for sub_step in step.generate_steps():
+            execute_step(sub_step, scenario_data)
+    else:
         step.execute()
         scenario_data.save()
 
