@@ -6,12 +6,12 @@ This module contains some utilities functions for the execution sub package
 import os
 from typing import Any, List, Optional, Tuple
 
-from multiversx_sdk_cli.accounts import Address
 from multiversx_sdk_cli.contracts import QueryResult, SmartContract
-from multiversx_sdk_cli.errors import BadAddressFormatError
+from multiversx_sdk_core.address import Address
+from multiversx_sdk_core.errors import ErrBadAddress
 
 from mxops.config.config import Config
-from mxops.data.data import ScenarioData
+from mxops.data.execution_data import ScenarioData
 from mxops import errors
 from mxops.execution.account import AccountsManager
 
@@ -22,15 +22,15 @@ def retrieve_specified_type(arg: str) -> Tuple[str, Optional[str]]:
     Example:
         $MY_VAR:int
         &MY_VAR:str
-        %CONTRACT%ID%MY_VAR:int
+        %KEY_1.KEY_2[0].MY_VAR:int
 
     :param arg: string arg passed
     :type arg: str
     :return: inner arg and name of the desired type if it exists
     :rtype: Tuple[str, Optional[str]]
     """
-    if ':' in arg:
-        return arg.split(':')
+    if ":" in arg:
+        return arg.split(":")
     return arg, None
 
 
@@ -46,9 +46,9 @@ def convert_arg(arg: Any, desired_type: Optional[str]) -> Any:
     :return: converted argument if a specified type was provided
     :rtype: Any
     """
-    if desired_type == 'str':
+    if desired_type == "str":
         return str(arg)
-    if desired_type == 'int':
+    if desired_type == "int":
         return int(arg)
     return arg
 
@@ -62,10 +62,13 @@ def retrieve_value_from_env(arg: str) -> str:
     :return: value saved in the environment
     :rtype: str
     """
-    if not arg.startswith('$'):
-        raise ValueError(f'the argument as no $ sign: {arg}')
+    if not arg.startswith("$"):
+        raise ValueError(f"the argument as no $ sign: {arg}")
     inner_arg, desired_type = retrieve_specified_type(arg)
-    retrieved_value = os.environ[inner_arg[1:]]
+    try:
+        retrieved_value = os.environ[inner_arg[1:]]
+    except KeyError as err:
+        raise errors.UnkownVariable(inner_arg[1:]) from err
     return convert_arg(retrieved_value, desired_type)
 
 
@@ -78,36 +81,37 @@ def retrieve_value_from_config(arg: str) -> str:
     :return: value saved in the config
     :rtype: str
     """
-    if not arg.startswith('&'):
-        raise ValueError(f'the argument as no & sign: {arg}')
-    inner_arg, desired_type = retrieve_specified_type(arg)
+    if not arg.startswith("&"):
+        raise ValueError(f"the argument has no & sign: {arg}")
+    inner_arg, desired_type = retrieve_specified_type(arg[1:])
     config = Config.get_config()
-    retrieved_value = config.get(inner_arg[1:].upper())
+    retrieved_value = config.get(inner_arg.upper())
     return convert_arg(retrieved_value, desired_type)
 
 
 def retrieve_value_from_scenario_data(arg: str) -> str:
     """
     Retrieve the value of an argument from scenario data.
-    the argument must formated like this: %<contract_id>%<attribute>
+    the argument must start with '%' and can chain key and index values:
+        - "%contract_id.address"
+        - "my_random_values.times[5]"
+        - "key_1.key_2[20].data"
 
     :param arg: name of the variable formated as above
     :type arg: str
     :return: value saved in the config
     :rtype: str
     """
-    inner_arg, desired_type = retrieve_specified_type(arg)
-    try:
-        contract_id, value_key = inner_arg[1:].split('%')
-    except Exception as err:
-        raise errors.WrongScenarioDataReference from err
+    if not arg.startswith("%"):
+        raise ValueError(f"the argument has no % sign: {arg}")
+    inner_arg, desired_type = retrieve_specified_type(arg[1:])
 
     scenario_data = ScenarioData.get()
-    retrieved_value = scenario_data.get_contract_value(contract_id, value_key)
+    retrieved_value = scenario_data.get_value(inner_arg)
     return convert_arg(retrieved_value, desired_type)
 
 
-def retrieve_address_from_account(arg: str) -> str:
+def retrieve_address_from_account(arg: str) -> Address:
     """
     Retrieve an address from the accounts manager.
     the argument must formated like this: [user]
@@ -115,7 +119,7 @@ def retrieve_address_from_account(arg: str) -> str:
     :param arg: name of the variable formated as above
     :type arg: str
     :return: address from the scenario
-    :rtype: str
+    :rtype: Address
     """
     try:
         arg = arg[1:-1]
@@ -123,7 +127,7 @@ def retrieve_address_from_account(arg: str) -> str:
         raise errors.WrongScenarioDataReference from err
 
     account = AccountsManager.get_account(arg)
-    return account.address.bech32()
+    return account.address
 
 
 def retrieve_value_from_string(arg: str) -> Any:
@@ -136,20 +140,56 @@ def retrieve_value_from_string(arg: str) -> Any:
     :return: untouched argument or retrieved value
     :rtype: Any
     """
-    if arg.startswith('['):
-        return retrieve_address_from_account(arg)
-    if arg.startswith('$'):
+    if arg.startswith("0x"):
+        return bytes.fromhex(arg[2:])
+    if arg.startswith("["):
+        return retrieve_address_from_account(arg).bech32()
+    if arg.startswith("$"):
         return retrieve_value_from_env(arg)
-    if arg.startswith('&'):
+    if arg.startswith("&"):
         return retrieve_value_from_config(arg)
-    if arg.startswith('%'):
+    if arg.startswith("%"):
         return retrieve_value_from_scenario_data(arg)
     return arg
 
 
+def retrieve_values_from_strings(args: List[str]) -> List[Any]:
+    """
+    Dynamically evaluate the value of each element of the provided list
+
+    :param args: args to evaluate
+    :type args: List[str]
+    :return: evaluated arguments
+    :rtype: List[Any]
+    """
+    return [retrieve_value_from_string(arg) for arg in args]
+
+
+def retrieve_value_from_any(arg: Any) -> Any:
+    """
+    Dynamically evaluate the provided argument depending on its type.
+    Otherwise it will be returned itself
+
+    :param arg: argument to evaluate
+    :type arg: Any
+    :return: evaluated argument
+    :rtype: Any
+    """
+    if isinstance(arg, str):
+        return retrieve_value_from_string(arg)
+    if isinstance(arg, list):
+        return [retrieve_value_from_any(e) for e in arg]
+    if isinstance(arg, dict):
+        return {
+            retrieve_value_from_any(k): retrieve_value_from_any(v)
+            for k, v in arg.items()
+        }
+    return arg
+
+
 def format_tx_arguments(arguments: List[Any]) -> List[Any]:
     """
-    Transform the arguments so they can be recognised by mxpy
+    Transform the arguments so they can be recognised by multiversx sdk core
 
     :param arguments: list of arguments to be supplied to a endpoint
     :type arguments: List[Any]
@@ -158,21 +198,30 @@ def format_tx_arguments(arguments: List[Any]) -> List[Any]:
     """
     formated_arguments = []
     for arg in arguments:
-        if isinstance(arg, str):  # done a first time as int arg can be entered as string
+        # convert a first time as int arg can be entered as string
+        if isinstance(arg, str):
             arg = retrieve_value_from_string(arg)
         formated_arg = arg
         if isinstance(arg, str):
-            if arg.startswith('erd') and len(arg) == 62:
-                formated_arg = '0x' + Address(arg).hex()
-            elif not arg.startswith('0x'):
-                formated_arg = 'str:' + arg
-        elif isinstance(arg, Address):
-            formated_arg = '0x' + arg.hex()
-
+            if arg.startswith("erd") and len(arg) == 62:
+                formated_arg = Address.from_bech32(arg)
         formated_arguments.append(formated_arg)
     return formated_arguments
 
 
+def retrieve_and_format_arguments(arguments: List[Any]) -> List[Any]:
+    """
+    Retrieve the MxOps value of the arguments if necessary and transform them
+    to match multiversx sdk core format
+
+    :param arguments: lisf of arguments to be supplied
+    :type arguments: List[Any]
+    :return: format args
+    :rtype: List[Any]
+    """
+    return format_tx_arguments(retrieve_value_from_any(arguments))
+
+
 def get_contract_instance(contract_str: str) -> SmartContract:
     """
     From a string return a smart contract instance.
@@ -186,22 +235,58 @@ def get_contract_instance(contract_str: str) -> SmartContract:
     """
     # try to see if the string is a valid address
     try:
-        return SmartContract(Address(contract_str))
-    except BadAddressFormatError:
+        return SmartContract(Address.from_bech32(contract_str))
+    except ErrBadAddress:
         pass
     # otherwise try to parse it as a mxops value
     contract_address = retrieve_value_from_string(contract_str)
     try:
-        return SmartContract(Address(contract_address))
-    except BadAddressFormatError:
+        return SmartContract(Address.from_bech32(contract_address))
+    except ErrBadAddress:
         pass
     # lastly try to see if it is a valid contract id
-    contract_address = retrieve_value_from_string(f'%{contract_str}%address')
+    contract_address = retrieve_value_from_string(f"%{contract_str}.address")
     try:
-        return SmartContract(Address(contract_address))
-    except BadAddressFormatError:
+        return SmartContract(Address.from_bech32(contract_address))
+    except ErrBadAddress:
         pass
-    raise errors.ParsingError(contract_str, 'contract address')
+    raise errors.ParsingError(contract_str, "contract address")
+
+
+def get_address_instance(address_str: str) -> Address:
+    """
+    From a string return an Address instance.
+    The input will be parsed to dynamically evaluate values from the environment,
+    the config, saved data or from the defined contracts or accounts.
+
+    :param address_str: raw address or address entity designation
+    :type address_str: str
+    :return: address instance corresponding to the input
+    :rtype: Address
+    """
+    # try to parse it as a mxops value
+    evaluated_address_str = retrieve_value_from_string(address_str)
+
+    # try to see if the string is a valid address
+    try:
+        return Address.from_bech32(evaluated_address_str)
+    except ErrBadAddress:
+        pass
+
+    # else try to see if it is a valid contract id
+    try:
+        evaluated_address_str = retrieve_value_from_string(f"%{address_str}.address")
+        return Address.from_bech32(evaluated_address_str)
+    except (ErrBadAddress, errors.WrongDataKeyPath):
+        pass
+
+    # finally try to see if it designates a defined account
+    try:
+        account = AccountsManager.get_account(evaluated_address_str)
+        return account.address
+    except errors.UnknownAccount:
+        pass
+    raise errors.ParsingError(address_str, "address_str address")
 
 
 def parse_query_result(result: QueryResult, expected_return: str) -> Any:
@@ -215,8 +300,8 @@ def parse_query_result(result: QueryResult, expected_return: str) -> Any:
     :return: parsed result of the query
     :rtype: Any
     """
-    if expected_return == 'number':
+    if expected_return in ("number", "int"):
         return result.number
-    if expected_return == 'str':
+    if expected_return == "str":
         return bytes.fromhex(result.hex).decode()
-    raise ValueError(f'Unkown expected return: {expected_return}')
+    raise ValueError(f"Unkown expected return: {expected_return}")
