@@ -9,7 +9,7 @@ from mxops import errors
 
 def test_out_of_gas(test_data_folder_path: Path):
     # Given
-    with open(test_data_folder_path / 'api_responses' / 'out_of_gas.json') as file:
+    with open(test_data_folder_path / "api_responses" / "out_of_gas.json") as file:
         tx = TransactionOnNetwork.from_proxy_http_response(**json.load(file))
 
     # When
@@ -17,14 +17,14 @@ def test_out_of_gas(test_data_folder_path: Path):
     # Then
     try:
         raise_on_errors(tx)
-        raise RuntimeError('`InternalVmExecutionError` was expected but was not raised')
+        raise RuntimeError("`InternalVmExecutionError` was expected but was not raised")
     except errors.InternalVmExecutionError:
         pass
 
 
 def test_not_enough_esdt(test_data_folder_path: Path):
     # Given
-    with open(test_data_folder_path / 'api_responses' / 'not_enough_esdt.json') as file:
+    with open(test_data_folder_path / "api_responses" / "not_enough_esdt.json") as file:
         tx = TransactionOnNetwork.from_proxy_http_response(**json.load(file))
 
     # When
@@ -32,14 +32,14 @@ def test_not_enough_esdt(test_data_folder_path: Path):
     # Then
     try:
         raise_on_errors(tx)
-        raise RuntimeError('`InvalidTransactionError` was expected but was not raised')
+        raise RuntimeError("`InvalidTransactionError` was expected but was not raised")
     except errors.InvalidTransactionError:
         pass
 
 
 def test_vm_error(test_data_folder_path: Path):
     # Given
-    with open(test_data_folder_path / 'api_responses' / 'vm_error.json') as file:
+    with open(test_data_folder_path / "api_responses" / "vm_error.json") as file:
         tx = TransactionOnNetwork.from_proxy_http_response(**json.load(file))
 
     # When
@@ -47,6 +47,6 @@ def test_vm_error(test_data_folder_path: Path):
     # Then
     try:
         raise_on_errors(tx)
-        raise RuntimeError('`InternalVmExecutionError` was expected but was not raised')
+        raise RuntimeError("`InternalVmExecutionError` was expected but was not raised")
     except errors.InternalVmExecutionError:
         pass
